From: Mike Snitzer <snitzer@redhat.com>

If an mpath device is held open when all paths in the last priority
group have failed userspace multipathd will attempt to reload the
associated DM table to reflect that the device no longer has any
priority groups.  But the reload attempt always failed because the
multipath target did not allow 0 priority groups.

Adjust multipath target to allow a table with both 0 priority groups
and 0 for the initial priority group number.

All multipath target messages related to priority group (enable_group,
disable_group, switch_group) will properly handle a priority group of
0 (will cause error).

When reloading a multipath table with 0 priority groups, userspace
multipathd must be updated to specify an initial priority group number
of 0 (rather than 1).

Signed-off-by: Mike Snitzer <snitzer@redhat.com>
Signed-off-by: Alasdair G Kergon <agk@redhat.com>
Cc: Babu Moger <babu.moger@lsi.com>
Acked-by: Hannes Reinecke <hare@suse.de>

---
drivers/md/dm-mpath.c |   13 ++++++++++---
 drivers/md/dm-mpath.c |   13 ++++++++++---
 1 file changed, 10 insertions(+), 3 deletions(-)

Index: linux-2.6.38-rc4/drivers/md/dm-mpath.c
===================================================================
--- linux-2.6.38-rc4.orig/drivers/md/dm-mpath.c
+++ linux-2.6.38-rc4/drivers/md/dm-mpath.c
@@ -844,8 +844,8 @@ static int multipath_ctr(struct dm_targe
 {
 	/* target parameters */
 	static struct param _params[] = {
-		{1, 1024, "invalid number of priority groups"},
-		{1, 1024, "invalid initial priority group number"},
+		{0, 1024, "invalid number of priority groups"},
+		{0, 1024, "invalid initial priority group number"},
 	};
 
 	int r;
@@ -879,6 +879,13 @@ static int multipath_ctr(struct dm_targe
 	if (r)
 		goto bad;
 
+	if ((!m->nr_priority_groups && next_pg_num) ||
+	    (m->nr_priority_groups && !next_pg_num)) {
+		ti->error = "invalid initial priority group";
+		r = -EINVAL;
+		goto bad;
+	}
+
 	/* parse the priority groups */
 	while (as.argc) {
 		struct priority_group *pg;
@@ -1417,7 +1424,7 @@ static int multipath_status(struct dm_ta
 	else if (m->current_pg)
 		pg_num = m->current_pg->pg_num;
 	else
-			pg_num = 1;
+		pg_num = (m->nr_priority_groups ? 1 : 0);
 
 	DMEMIT("%u ", pg_num);
 
